# TerrainProcessing.py
#
# This tool runs basic terrain analysis procedures on an input elevation dataset.  The script is
# designed to work on a LIDAR-derived DEM of METER horizontal resolution.
# The following procedures will be run on the input DEM:
# 1)Fill 2)FlowDirection 3)FlowAccumulation(optional), and 4)Hillshade(optional)
#
##--------------------------------------------------------------------------------
## Original coding: S. Porter - NLAE 04/2014
##--------------------------------------------------------------------------------
#
##--------------------------------------------------------------------------------
# Import System Modules
##--------------------------------------------------------------------------------
import arcpy
from arcpy import env
from arcpy.sa import *
import sys, string, os, time
arcpy.CheckOutExtension("Spatial")


# Modules -----------------------------------------------------------------------_

def TerrainProcessing(DEM, ZFactor):

    # Fill the cut DEM
    arcpy.AddMessage("Filling DEM")
    DEMFill = Fill(DEM)
    DEMFill.save(OutputFillDEM)

    arcpy.BuildPyramids_management(OutputFillDEM)

    # Calculate Flow Direction
    arcpy.AddMessage("Calculating D8 Flow Direction")
    D8FlowDir = FlowDirection(DEMFill, "", "")
    D8FlowDir.save(OutputFlowDir)

    arcpy.BuildPyramids_management(OutputFlowDir)
    
    # Calculate Flow Accumulation
    arcpy.AddMessage("Calculating Flow Accumulation")
    D8Accumulation = FlowAccumulation(D8FlowDir)
    D8Accumulation.save(OutputFlowAcc)

    arcpy.BuildPyramids_management(OutputFlowAcc)

    # Calculate Hillshade
    arcpy.AddMessage("Generating Hillshade")
    HillshadeOut = Hillshade(DEM, "", "", "", ZFactor)
    HillshadeOut.save(OutputHshd)

    arcpy.BuildPyramids_management(OutputHshd)
  

##------------------------------------------------------------------------------
##------------------------------------------------------------------------------
if __name__ == "__main__":

    # Define parameters	
    DEM = arcpy.GetParameterAsText(0)
    OutputFillDEM = arcpy.GetParameterAsText(1)
    OutputFlowDir = arcpy.GetParameterAsText(2)
    OutputFlowAcc = arcpy.GetParameterAsText(3)
    OutputHshd = arcpy.GetParameterAsText(4)
    ZFactor = arcpy.GetParameterAsText(5)

    # Set environments
    arcpy.env.extent = DEM
    arcpy.env.snapRaster = DEM
    rDesc = arcpy.Describe(DEM)
    arcpy.env.cellSize = rDesc.meanCellHeight
    arcpy.env.overwriteOutput = True
    arcpy.env.scratchWorkspace = env.scratchFolder

    if not arcpy.Exists(env.workspace):
	arcpy.AddError("workspace does not exist!! Please set your workspace to a valid path directory in Arcmap --> Geoprocessing --> Environments --> Workspace")
	sys.exit(0)

    # Run Modules
    TerrainProcessing(DEM, ZFactor)
