# Identifies depression in an input DEM.  The first step is to fill the input DEM, then
# subtract the unfilled DEM from the filled DEM.  Differences of 0 or less than 0 are converted
# to null, and sink values are replaced with the difference in elevation, which represents depth
#
#
##--------------------------------------------------------------------------------
## Original coding: S. Porter - NLAE 08/2015
##--------------------------------------------------------------------------------
#
##--------------------------------------------------------------------------------
# Import System Modules
##--------------------------------------------------------------------------------
import arcpy
from arcpy import env
from arcpy.sa import *
import sys, string, os, time


def ImpededFlow(DEM):
    arcpy.AddMessage("Filling DEM")
    DEMFill = Fill(DEM)

    # Subtract the unfilled DEM from the filled DEM
    arcpy.AddMessage("Subtracting unfilled DEM from filled DEM")
    FillReg = Minus(DEMFill, DEM)

    DepthGrid = Con(FillReg, FillReg, "", "Value > 0")
    DepthGrid.save(OutDepthGrid)

    arcpy.BuildPyramids_management(OutDepthGrid)

    arcpy.Delete_management(FillReg)
    arcpy.Delete_management(DEMFill)
    del[FillReg, DEMFill]
    

##------------------------------------------------------------------------------
##------------------------------------------------------------------------------
if __name__ == "__main__":

    # Define parameters	
    DEM = arcpy.GetParameterAsText(0)
    OutDepthGrid = arcpy.GetParameterAsText(1)

    # Set environments
    arcpy.env.extent = DEM
    arcpy.env.snapRaster = DEM
    rDesc = arcpy.Describe(DEM)
    arcpy.env.cellSize = rDesc.meanCellHeight
    arcpy.env.overwriteOutput = True

    # Run Modules
    ImpededFlow(DEM)
